// Included Header Files

#include "Creature.h"



/*	**********************************************************
	* Attack                                                 *
	* ------                                                 *
	* Handles the process of attacking.  This includes       *
	* accuracy, avoidance, damage mitigation, and damage     *
	* calculation.                                           *
	**********************************************************	*/

void Creature::Attack(Creature *other) {
	float toHit = 0.0f;
	int damageMitigation = 0;
	int numberOfAttacks = 1;
	short currentHand = primaryHand;
	float chanceToDualWield, chanceToBeat;

	for (int n = 0; n < numberOfAttacks; n++) {
		toHit = 0.0f;
		damageMitigation = 0;

		// accuracy
		toHit = AttackAccuracy(other, currentHand);

		// avoidance
		damageMitigation = DamageAvoidanceEvasion(other, toHit);
		// TODO - let the numbers returned from Evasion() mean something

		// delay goes here (maybe after dual wield check)
		// if dual wield activates then there should be less delay
		// depending on skill between the primary & secondary
		// attacks

		// dual wield
		if (dualWield.level < 50) {
			chanceToDualWield = (float) (rand() % 100) * (float) dualWield.level;
			chanceToBeat = (float) (rand() % 100) * 150.0f;
		} else if (dualWield.level < 100) {
			chanceToDualWield = (float) (rand() % 100) * (float) dualWield.level;
			chanceToBeat = (float) (rand() % 100) * 100.0f;
		} else if (dualWield.level < 150) {
			chanceToDualWield = (float) (rand() % 100) * (float) dualWield.level;
			chanceToBeat = (float) (rand() % 100) * 50.0f;
		} else {
			chanceToDualWield = (float) (rand() % 100) * (float) dualWield.level;
			chanceToBeat = (float) (rand() % 100) * 10.0f;
		};
		if (chanceToDualWield > chanceToBeat && numberOfAttacks < 2) {
			printf("executing attack with secondary hand\n");
			numberOfAttacks++;
			currentHand = secondaryHand;
		};
	};

	printf("Attacker's evasion: %d\n", this->evasion.level);
	printf("Defender's evasion: %d\n", other->evasion.level);
	printf("\n");
};



/*	**********************************************************
	* Attack Accuracy                                        *
	* ---------------                                        *
	* The chance that the attacker has to hit the target.    *
	**********************************************************	*/

float Creature::AttackAccuracy(Creature *other, short whichHand) {
	Creature* attacker = this;
	Creature* defender = other;
	float toHit = 0.0f;
	Size weaponSize = AVERAGE;

	// PRIMARY WEAPON

	// Modifier:  Dexterity (moderate effect)
	toHit += (float) attacker->dexterity / 5;

	// Modifier:  Weapon Skill (highest effect)
	toHit += (float) attacker->GetWeaponSkill(whichHand);

	// Modifier:  Weapon size (small effect)
	if (whichHand == primaryHand) {
		weaponSize = attacker->GetPrimaryWeapon().GetSize();
	} else {
		weaponSize = attacker->GetSecondaryWeapon().GetSize();
	};
	switch (weaponSize) {
		case TINY:
			toHit += 4.0f;
			break;
		case SMALL:
			toHit += 2.0f;
			break;
		case AVERAGE:
			toHit += 0.0f;
			break;
		case LARGE:
			toHit += -2.0f;
			break;
		case MASSIVE:
			toHit += -4.0f;
			break;
		default:
			toHit += 0;
			break;
	};

	// Result
	return toHit;
};



/*	**********************************************************
	* Damage Avoidance (Evasion)                             *
	* --------------------------                             *
	* This calculates the chance the defender will evade the *
	* attack and returns a number representing how well the  *
	* defender evaded that attack (partial evasion results   *
	* in minor damage mitigation).                           *
	**********************************************************	*/

int Creature::DamageAvoidanceEvasion(Creature *other, float toHitChance) {
	Creature *attacker = this;
	Creature *defender = other;
	float toHit = toHitChance;
	float evasionChance = 0.0f;
	float hindrance, encumbrance;

	// Modifier:  Evasion (highest effect)
	evasionChance += (float) defender->evasion.level;

	// Modifier:  Agility (moderate effect)
	evasionChance += (float) defender->agility / 5;

	// Modifier:  Weight (small effect)
	evasionChance -= (float) defender->weight / 10;

	// Modifier:  Size (very small effect)
	switch (defender->GetSize()) {
		case TINY:
			evasionChance += 4.0f;
			break;
		case SMALL:
			evasionChance += 2.0f;
			break;
		case AVERAGE:
			evasionChance += 0.0f;
			break;
		case LARGE:
			evasionChance += -2.0f;
			break;
		case MASSIVE:
			evasionChance += -4.0f;
			break;
		default:
			evasionChance += 0.0f;
			break;
	};

	// Modifier:  Hindrance (moderate effect)
	hindrance = (float) (defender->inventory.GetTotalArmorWeight() * 10)
			  - armorUsage.level
			  - defender->strength;
	if (hindrance < 0.0f) {
		hindrance = 0.0f;
	};
	evasionChance -= hindrance;

	// Modifier:  Encumbrance (small effect)
	encumbrance = (float) defender->inventory.GetTotalWeight()
				- defender->strength;
	if (encumbrance < 0.0f) {
		encumbrance = 0.0f;
	};
	evasionChance -= encumbrance;

	// Modifier:  Randomness & Reduction
	evasionChance *= (float) rand();
	toHit *= (float) rand();

	// Result:  100% dodge
	if (evasionChance / 5.0f > toHit) {
		printf("You evaded an attack!\n");
		this->LearnSkill(&defender->evasion);
		return 4;
	// Result:  75% dodge
	} else if (evasionChance / 4.0f > toHit) {
		printf("You nearly evaded the attack.\n");
		this->LearnSkill(&defender->evasion);
		return 3;
	// Result:  50% dodge
	} else if (evasionChance / 3.0f > toHit) {
		printf("You attempted to evade, but still got nicked.\n");
		this->LearnSkill(&defender->evasion);
		return 2;
	// Result:  25% dodge
	} else if (evasionChance / 2.0f > toHit) {
		printf("You attempted to evade, but took most of the blow.\n");
		this->LearnSkill(&defender->evasion);
		return 1;
	// Result:  0% dodge
	} else {
		return 0;
	};

	return 0;
};

	// Modifier:  Riposte
	// chance to avoid damage and do definite damage back

	// Modifier:  Blocking with a Shield
	// 1. blocking with a shield can avoid damage
	// 2. partial blocking can mitigate damage (more than parrying)

	// Modifier:  Parry
	// 1. parry can avoid damage
	// 2. partial parry can mitigate damage (more than evading)


// TODO:
// Defense portion - order of chances
// 1. Evasion
// 2. Riposte (evade, and counter-attack)
// 3. Shield Block
// 3. Parry
// 4. Block
// 5. Armor

// TODO
// dual wield
// At a certain level, using both weapons should almost always occur
// At higher levels, this should increase accuracy/power of the secondary weapon
//         as well as attack speed



//	Melee Mitigation: check damage resistance ability of defender if the hit
//                       was successful
// blocking:
// only reduces damage based on unarmoredResistance if user is wearing no armor
// (and a little damage reduction, since it avoids getting hit in the head,
// chest, etc.), otherwise it looks at armor skill and the armor stats itself,
// also can block with a shield and .... ???

// NO! (tested in real life)
//    The strength of the attacker & weight of the weapon, the right ratio
//    plays an important part of attack speed, but also landing a hit
