// Included Header Files

#include "Creature.h"
#include <stdlib.h>    // rand()
#include <math.h>



/*	**********************************************************
	* Creature                                               *
	* --------                                               *
	* Constructor (initialization)                           *
	**********************************************************	*/

Creature::Creature() {
	inventory.GenerateWeapon(); // TEMPORARY

	// Initialize general attributes

	primaryHand = RIGHT_HAND;
	secondaryHand = LEFT_HAND;
	size = SMALL;
	weight = 10;

	// Initialize defensive skills

	armorUsage.name = "Armor Usage";
	armorUsage.level = 0;
	armorUsage.description = "the ability to manuever in armor";
	evasion.name = "Evasion";
	evasion.level = 0;
	evasion.description = "the ability to evade attacks";
	block.name = "Blocking";
	block.level = 0;
	block.description = "the ability to block attacks";
	shield.name = "Shield Usage";
	shield.level = 0;
	shield.description = "the ability to block with a shield";
	parry.name = "Parrying";
	parry.level = 0;
	parry.description = "the ability to parry with a weapon";
	riposte.name = "Riposte";
	riposte.level = 0;
	riposte.description = "the ability to riposte attacks";
	multipleOpponent.name = "Multiple Opponent";
	multipleOpponent.level = 0;
	multipleOpponent.description = "the ability to handle multiple engaged enemies";

	// Initialize offensive skills

	unarmed.name = "Unarmed";
	unarmed.level = 0;
	unarmed.description = "the ability to attack with your fists";
	smallPiercing.name = "Small Piercing";
	smallPiercing.level = 0;
	smallPiercing.description = "the ability to use small piercing weapons";
	dualWield.name = "Dual Wield";
	dualWield.level = 0;
	dualWield.description = "the ability to wield two weapons simultaneously";
};



/*	**********************************************************
	* ~Creature                                              *
	* ---------                                              *
	* Destructor (memory deallocation)                       *
	**********************************************************	*/

Creature::~Creature() {
};



/*	**********************************************************
	* Set Creature Stats                                     *
	* ------------------                                     *
	* Sets every ability of the creature to a given number.  *
	**********************************************************	*/

void Creature::setCreatureStats(short skillLevel) {
	strength = skillLevel;
	dexterity = skillLevel;
	agility = skillLevel;

	unarmoredResistance.level = skillLevel;
	armorUsage.level = skillLevel;
	evasion.level = skillLevel;
	block.level = skillLevel;
	shield.level = skillLevel;
	parry.level = skillLevel;
	riposte.level = skillLevel;
	multipleOpponent.level = skillLevel;

	unarmed.level = skillLevel;
	smallPiercing.level = skillLevel;
	dualWield.level = skillLevel;
};



/*	**********************************************************
	* Get Weapon Skill                                       *
	* ----------------                                       *
	* Acquires the weapon skill of the creature provided     *
	* which hand the weapon is in.                           *
	**********************************************************	*/

short Creature::GetWeaponSkill(short whichHand) {
	Item weapon;

	// check which hand is being asked for and get the item of it
	if (whichHand == RIGHT_HAND) {
		weapon = inventory.GetPrimaryHand();
	} else if (whichHand == LEFT_HAND) {
		weapon = inventory.GetSecondaryHand();
	} else {
		return 0;
	};

	// check what type of weapon it is and return the skill of the weapon
	switch (weapon.GetType()) {
		case UNARMED:
			return unarmed.level;
		case SMALL_PIERCING:
			return smallPiercing.level;
		default:
			return 0;
	};

	return 0;
};



/*	**********************************************************
	* Learn Skill                                            *
	* -----------                                            *
	* This gets called after the use of any skill and has a  *
	* chance of that skill gaining a point.                  *
	**********************************************************	*/

void Creature::LearnSkill(Skill *skill) {
	// TODO: This needs to factor in differences of combat abilities
	//       of attacker/defender... doesn't it?  Think about it.
	// still working on the equation... this one is OKAY at the moment

	double chanceToLearn = (double) rand() / skill->level;
	//double chanceToLearn = (double) rand() / sqrt((double) skill->level);
	double randomChance = (double) rand() * (double) skill->level;

	// creature has gained another rank
	if (chanceToLearn > randomChance) {
		printf("You have become better in %s", skill->description);
		printf("!\n");
		skill->level++;
	};
};